/* libdf3 - error.c
 * nb buffer size hardwired.
 * nb may get confused by uninitialised (df3).
 */

#include "df3_priv.h"

#include <string.h>


static char const * const messages[] = {
  /* ecOk */           "no error",
  /* ecBadArgument */  "bad function argument",
  /* ecBadDimsAxes */  "bad axes value(s)",
  /* ecBadDimsBpv */   "bad bpv",
  /* ecBadHandle */    "bad handle",
  /* ecBadVoxAt */     "coord value(s) too large",
  /* ecBadVoxVal */    "voxel value too large",
  /* ecNotDf3file */   "not df3 file",
  /* ecUnusablefile */ "no (usable) file",
  /* ecSysErr */       "lib/sys error",
  /* ecLastEC */       "OOPS. ought not display."
};


char const *df3_error_message(df3Error const ec) {
  return messages[(size_t)ec];
}


char const *df3_handle_error(df3Handle *df3, bool const clear) {
  static char lastemsg[1024];
  bool const f = df3ValidHandle(df3);

  if (!df3)
    strcpy(lastemsg, messages[(size_t)ecBadHandle]);
  else if (ecSysErr == df3->ec)
    snprintf(lastemsg, 1024, "%svalid handle, errno %d (%s)",
            f ? "" : "in",
            df3->syserr, strerror(df3->syserr));
  else
    snprintf(lastemsg, 1024, "%svalid handle, error %d (%s)",
            f ? "" : "in",
            df3->ec, messages[(size_t)df3->ec]);

  if (clear  &&  f) {
    df3->syserr = 0;
    df3->ec = ecOk;
  }

  return lastemsg;
}

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
